using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using Castle.Core.Resource;
using Castle.Windsor;
using Castle.Windsor.Configuration.Interpreters;
using employee.system.domain;
using employee.system.infrastructure.configuration;
using employee.system.infrastructure.configuration.custom;
using employee.system.infrastructure.containers;
using employee.system.infrastructure.mapping;
using employee.system.orm.data;
using employee.system.orm.mapping;
using employee.system.orm.repository;
using employee.system.orm.repository.custom;
using log4net;

namespace employee.system.runners
{
    public class ViewerConsoleRunner : IConsoleRunner
    {
        private static readonly ILog _logger = LogManager.GetLogger(typeof (ViewerConsoleRunner));

        public void run_the_console()
        {
            IEnumerable<IEmployee> employees = null;

            employees = example_with_dependency_injection();
            //employees = example_with_windsor_container();
            //employees = example_with_windsor_container_configured_by_xml();

            foreach (IEmployee employee in employees)
            {
                _logger.InfoFormat(employee.ToString());
            }
        }

        private static IEnumerable<IEmployee> example_with_dependency_injection()
        {
            ISystemConfiguration config = new NorthwindConfiguration();
            //config = new AdventureWorksConfiguration();

            IDbConnection connection = new SqlConnection();
            IMapper<DataRow, IEmployee> mapper = new MapFromDataRowToEmployee();
            IDataAccess data_accessor = new DataAccess(connection, config);

            EmployeeRepository the_employee_repository = new EmployeeRepository(data_accessor, mapper);

            return the_employee_repository.get_employees();
        }

        private static IEnumerable<IEmployee> example_with_windsor_container()
        {
            IWindsorContainer the_container = new WindsorContainer();
            //the_container.AddComponent("nwConfig", typeof (ISystemConfiguration), typeof (NorthwindConfiguration));
            the_container.AddComponent("awConfig", typeof (ISystemConfiguration), typeof (AdventureWorksConfiguration));
            the_container.AddComponent("data", typeof (IDataAccess), typeof (DataAccess));
            the_container.AddComponent("repository.simple", typeof (IRepository), typeof (SimpleRepository));
            the_container.AddComponent("sqlconn", typeof (IDbConnection), typeof (SqlConnection));
            the_container.AddComponent("mapper.IDataReader-Employee",
                                       typeof (IMapper<DataRow, IEmployee>),
                                       typeof (MapFromDataRowToEmployee)
                );

            Container.initialize_with(the_container);

            IRepository local_repository = the_container.Resolve<IRepository>();
            //IRepository local_repository = Container.get_an_instance_of<IRepository>();
            return local_repository.a_list_of<IEmployee>();
            //return Repository.get().a_list_of<IEmployee>();
        }

        private static IEnumerable<IEmployee> example_with_windsor_container_configured_by_xml()
        {
            IWindsorContainer the_container = new WindsorContainer(new XmlInterpreter(new ConfigResource("castle")));
            Container.initialize_with(the_container);

            //return Container.get_an_instance_of<IRepository>().a_list_of<IEmployee>();
            return Repository.get().a_list_of<IEmployee>();
        }
    }
}