Imports System.Data
Imports System.Collections.Generic
Imports IoC.Library.Config
Imports IoC.Library.infrastructure.logging
Imports IoC.Library.infrastructure.mapping

Namespace DataUtility

    Public Class DataAccess
        Implements IDataAccess


        Private _dbConnection As IDbConnection
        Private _config As IConfig
        Private _mapper As IMapper(Of IDataReader, IEmployee)

        Public Sub New(ByVal dbConnection As IDbConnection, ByVal config As IConfig, ByVal mapper As IMapper(Of IDataReader, IEmployee))
            Log.bound_to(Me).DebugFormat("Initializing {0}.", Me.GetType.Name)
            _dbConnection = dbConnection
            _config = config
            _mapper = mapper

            _dbConnection.ConnectionString = config.DBConnString
        End Sub

        Public Function GetList(Of IEmployee)() As IList(Of IEmployee) Implements IDataAccess.GetList
            Dim list As IList(Of IEmployee) = New List(Of IEmployee)

            Log.bound_to(Me).Debug("DataAccess is creating a database command.")
            Using cmd As IDbCommand = _dbConnection.CreateCommand()

                cmd.CommandText = _config.GetCommand
                cmd.CommandType = CommandType.Text

                Log.bound_to(Me).Debug("DataAccess is opening a connection to the database.")
                _dbConnection.Open()
                Log.bound_to(Me).Debug("DataAccess is executing a data reader.")
                Using dr As IDataReader = cmd.ExecuteReader()
                    While dr.Read()
                        list.Add(CType(_mapper.MapFrom(dr), IEmployee))
                    End While
                End Using
                Log.bound_to(Me).Debug("DataAccess is closing the connection to the database.")
                If (_dbConnection.State = ConnectionState.Open) Then
                    _dbConnection.Close()
                End If
            End Using

            Log.bound_to(Me).InfoFormat("DataAccess returned {0} items.", list.Count)

            Return list
        End Function

        'Public Function GetList(Of Record)() As IList(Of Record) Implements IDataAccess.GetList
        '    Dim list As IList(Of Record) = New List(Of Record)

        '    Log.bound_to(Me).Debug("Creating a database command.")
        '    Using cmd As IDbCommand = _dbConnection.CreateCommand()

        '        cmd.CommandText = _config.GetCommand
        '        cmd.CommandType = CommandType.Text
        '
        '        Log.bound_to(Me).Debug("DataAccess is opening a connection to the database.")
        '        _dbConnection.Open()
        '        Log.bound_to(Me).Debug("DataAccess is executing a data reader.")
        '        Using dr As IDataReader = cmd.ExecuteReader()
        '            While dr.Read()
        '                list.Add(Map.from_type(Of IDataReader)(dr).to_type(of Record)())
        '            End While
        '        End Using
        '        Log.bound_to(Me).Debug("DataAccess is closing the connection to the database.")
        '        If (_dbConnection.State = ConnectionState.Open) Then
        '            _dbConnection.Close()
        '        End If
        '    End Using

        '    Log.bound_to(Me).InfoFormat("DataAccess returned {0} items.", list.Count)

        '    Return list
        'End Function

    End Class

End Namespace