Imports System.Collections.Generic
Imports System.Data.SqlClient
Imports System.Data
Imports Castle.Windsor.Configuration.Interpreters
Imports IoC.Library.infrastructure.mapping.custom
Imports IoC.Library.DataUtility
Imports IoC.Library.Config
Imports Castle.Windsor
Imports Castle.Core.Resource
Imports log4net
Imports IoC.Library.infrastructure.mapping

Public Class IoCProgram

    Private Shared ReadOnly _logger As ILog = LogManager.GetLogger(GetType(IoCProgram))

    Public Sub Run()
        Dim employees As IEnumerable(Of IEmployee)
        employees = DIExample()
        'employees = WindsorExample()
        'employees = WindsorConfigurationExample()

        For Each employee As IEmployee In employees
            _logger.InfoFormat(employee.ToString())
        Next
    End Sub

    Public Function DIExample() As IEnumerable(Of IEmployee)
        Dim config As IConfig = New NorthwindConfig
        'config = New AdventureWorksConfig
        Dim connection As IDbConnection = New SqlConnection()
        Dim mapper As IMapper(Of IDataReader, IEmployee) = New MapFromDataReaderToEmployee()
        Dim employeeInformation As EmployeeInformationRepository = New EmployeeInformationRepository(New DataAccess(connection, config, mapper))

        Return employeeInformation.GetEmployees()
    End Function

    Public Function WindsorExample() As IEnumerable(Of IEmployee)
        Dim container As IWindsorContainer = New WindsorContainer()

        container.AddComponent("nwConfig", GetType(IConfig), GetType(NorthwindConfig))
        container.AddComponent("awConfig", GetType(IConfig), GetType(AdventureWorksConfig))
        container.AddComponent("data", GetType(IDataAccess), GetType(DataAccess))
        container.AddComponent("emp-info", GetType(EmployeeInformationRepository))
        container.AddComponent("sqlconn", GetType(IDbConnection), GetType(SqlConnection))
        container.AddComponent("mapper.IDataReader-Employee", GetType(IMapper(Of IDataReader, IEmployee)), GetType(MapFromDataReaderToEmployee))

        'Dim EmployeeInformationRepository As EmployeeInformationRepository = CType(container(GetType(EmployeeInformationRepository)), EmployeeInformationRepository)
        Dim employeeInformation As EmployeeInformationRepository = container.Resolve(Of EmployeeInformationRepository)()

        Return employeeInformation.GetEmployees()
    End Function

    Public Function WindsorConfigurationExample() As IEnumerable(Of IEmployee)
        Dim container As IWindsorContainer = New WindsorContainer(New XmlInterpreter(New ConfigResource("castle")))

        Return container.Resolve(Of EmployeeInformationRepository)().GetEmployees()
    End Function

End Class